#include <SdFat.h>
#include <SdFatUtil.h>
#include <Ethernet.h>
#include <SPI.h>

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
byte ip[] = { 192, 168, 1, 177 };
EthernetServer server(80);

Sd2Card card;
SdVolume volume;
SdFile root;
SdFile file;

// store error strings in flash to save RAM
#define error(s) error_P(PSTR(s))

void error_P(const char* str) {
  PgmPrint("error: ");
  SerialPrintln_P(str);
  if (card.errorCode()) {
    PgmPrint("SD error: ");
    Serial.print(card.errorCode(), HEX);
    Serial.print(',');
    Serial.println(card.errorData(), HEX);
  }
  while(1);
}

void setup() {
  Serial.begin(9600);
 
  PgmPrint("Free RAM: ");
  Serial.println(FreeRam());  
  pinMode(10, OUTPUT);             //SS pin as an output (necessary!)
  digitalWrite(10, HIGH);               // W5100 chip

  if (!card.init(SPI_HALF_SPEED, 4)) error("card.init failed!");
  
  // инициализация FAT
  if (!volume.init(&card)) error("vol.init failed!");

  PgmPrint("Volume is FAT");
  Serial.println(volume.fatType(),DEC);
  Serial.println();
  
  if (!root.openRoot(&volume)) error("openRoot failed");

  // список файлов в корневой директории
  PgmPrintln("Files found in root:");
  root.ls(LS_DATE | LS_SIZE);

  // Recursive list of all directories
  PgmPrintln("Files found in all dirs:");
  root.ls(LS_R);
  
 PgmPrintln("Done");
  
  // запуск сервера
  Ethernet.begin(mac, ip);
  server.begin();
}
// вывод списка файлов
void ListFiles(EthernetClient client, uint8_t flags) {
  dir_t p;
root.rewind();
  client.println("<ul>");
  while (root.readDir(p) > 0) {
    if (p.name[0] == DIR_NAME_FREE) break;
if (p.name[0] == DIR_NAME_DELETED || p.name[0] == '.') continue;
    // вывод директорий и файлов
    if (!DIR_IS_FILE_OR_SUBDIR(&p)) continue;
    
    client.print("<li><a href=\"");
    for (uint8_t i = 0; i < 11; i++) {
      if (p.name[i] == ' ') continue;
      if (i == 8) {
        client.print('.');
      }
      client.print((char)p.name[i]);
    }
    client.print("\">");
    
    // печать имени файла
    for (uint8_t i = 0; i < 11; i++) {
      if (p.name[i] == ' ') continue;
      if (i == 8) {
        client.print('.');
      }
      client.print((char)p.name[i]);
    }
    
    client.print("</a>");
    
    if (DIR_IS_SUBDIR(&p)) {
      client.print('/');
    }

    // печать дата/время изменения файла
    if (flags & LS_DATE) {
       root.printFatDate(p.lastWriteDate);
       client.print(' ');
       root.printFatTime(p.lastWriteTime);
    }
    // вывод размера файла
    if (!DIR_IS_SUBDIR(&p) && (flags & LS_SIZE)) {
      client.print(' ');
      client.print(p.fileSize);
    }
    client.println("</li>");
  }
  client.println("</ul>");
}

#define BUFSIZ 100

void loop()
{
  char clientline[BUFSIZ];
  int index = 0;
  
  EthernetClient client = server.available();
  if (client) {
    // признак началь http запроса
    boolean current_line_is_blank = true;
    
    // сброс указателя позиции в буфере 
    index = 0;
    
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        
        // символы в буфер
        if (c != '\n' && c != '\r') {
          clientline[index] = c;
          index++;
          // не больше размера буфера
          if (index >= BUFSIZ) 
            index = BUFSIZ -1;
          
          // continue to read more data!
          continue;
        }
        
        // \n or \r – конец строки запроса
        clientline[index] = 0;
        
        // анализ строки запроса
        if (strstr(clientline, "GET / ") != 0) {
          // отправка в браузер заголовка
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println();
          
          // печать списка файлов
          client.println("<h2>Files:</h2>");
          ListFiles(client, LS_SIZE);
        } else if (strstr(clientline, "GET /") != 0) {
          // определение имени файла в запросе
          char *filename;
          
          filename = clientline + 5; // после "GET /" (5 символов)
          (strstr(clientline, " HTTP"))[0] = 0;
          
       if (! file.open(&root, filename, O_READ)) {
            client.println("HTTP/1.1 404 Not Found");
            client.println("Content-Type: text/html");
            client.println();
            client.println("<h2>File Not Found!</h2>");
            break;
          }
          
          Serial.println("Opened!");
                    
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/plain");
          client.println();
          
          int16_t c;
          while ((c = file.read()) > 0) {
             client.print((char)c);
          }
          file.close();
        } else {
          // страница 404
          client.println("HTTP/1.1 404 Not Found");
          client.println("Content-Type: text/html");
          client.println();
          client.println("<h2>File Not Found!</h2>");
        }
        break;
      }
    }
    // задержка для ожидания отправки данных в браузер
    delay(1);
    client.stop();
  }
}

